package cn.com.qmth.examcloud.core.oe.task.service.pipeline;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.helpers.KeyValuePair;
import cn.com.qmth.examcloud.commons.helpers.pipeline.NodeExecuter;
import cn.com.qmth.examcloud.commons.helpers.pipeline.TaskContext;
import cn.com.qmth.examcloud.core.oe.student.api.ExamRecordDataCloudService;
import cn.com.qmth.examcloud.core.oe.student.api.request.CalcExamScoreReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.CalcFaceBiopsyResultReq;
import cn.com.qmth.examcloud.core.oe.student.api.response.CalcExamScoreResp;
import cn.com.qmth.examcloud.core.oe.student.api.response.CalcFaceBiopsyResultResp;
import cn.com.qmth.examcloud.core.oe.task.dao.ExamCaptureQueueRepo;
import cn.com.qmth.examcloud.core.oe.task.service.ExamCaptureService;
import cn.com.qmth.examcloud.core.oe.task.service.ExamRecordDataService;
import cn.com.qmth.examcloud.core.oe.task.service.bean.CalculateFaceCheckResultInfo;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.SysPropertyCacheBean;
import cn.com.qmth.examcloud.support.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.support.enums.HandInExamType;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description 交卷后续处理执行器
 * @Author lideyin
 * @Date 2019/12/17 16:39
 * @Version 1.0
 */
@Component
public class AfterHandInExamExecutor implements NodeExecuter<Long, ExamRecordData, Long, ExamRecordData> {

    @Autowired
    private ExamRecordDataService examRecordDataService;
    private static Long DEFAULT_MAX_PROCESS_SECONDS = 30L;

    @Override
    public List<KeyValuePair<Long, ExamRecordData>> execute(Long key, ExamRecordData examRecordData, TaskContext context) throws Exception {

        List<KeyValuePair<Long, ExamRecordData>> resultList = new ArrayList<>();
        KeyValuePair<Long, ExamRecordData> keyValuePair = new KeyValuePair<>(key, examRecordData);

        //针对已交卷的数据进行交卷后续处理
        if (examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_HAND_IN ||
                examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_AUTO_HAND_IN) {

            SysPropertyCacheBean maxProcessSecondsProperty = CacheHelper.getSysProperty("oe.task.maxProcessSeconds");
            //本节点最大处理时长
            Long maxProcessSeconds;
            if (maxProcessSecondsProperty.getHasValue()) {
                maxProcessSeconds = Long.valueOf(maxProcessSecondsProperty.getValue().toString());
            } else {
                maxProcessSeconds = DEFAULT_MAX_PROCESS_SECONDS;
            }

            //交卷时间戳
            Long handInTime = (examRecordData.getEndTime() == null ? examRecordData.getCleanTime() : examRecordData.getEndTime()).getTime();
            Long times = System.currentTimeMillis() - handInTime;

            //如果交卷后超过指定时长内仍未处理完成，则交给下一节点进行处理
            if (times > maxProcessSeconds * 1000) {
                resultList.add(keyValuePair);
                return resultList;
            }

            examRecordData = examRecordDataService.processAfterHandInExam(examRecordData.getId());

            keyValuePair.setValue(examRecordData);
            resultList.add(keyValuePair);

            return resultList;
        }

        resultList.add(keyValuePair);
        return resultList;
    }

}
